@@ -20,10 +20,11 @@
 #include <dune/istl/solvers.hh>
 
 #include <dune/functions/functionspacebases/interpolate.hh>
-#include <dune/functions/functionspacebases/pqknodalbasis.hh>
+#include <dune/functions/functionspacebases/lagrangebasis.hh>
 #include <dune/functions/functionspacebases/raviartthomasbasis.hh>
-#include <dune/functions/functionspacebases/hierarchicvectorwrapper.hh>
+#include <dune/functions/backends/istlvectorbackend.hh>
 #include <dune/functions/functionspacebases/compositebasis.hh>
+#include <dune/functions/functionspacebases/subspacebasis.hh>
 #include <dune/functions/gridfunctions/discreteglobalbasisfunction.hh>
 #include <dune/functions/gridfunctions/gridviewfunction.hh>
 
@@ -48,7 +49,7 @@ void getLocalMatrix(const LocalView& localView,
   elementMatrix = 0;      // fills the entire matrix with zeroes
 
   // Get set of shape functions for this element
-  using namespace Dune::TypeTree::Indices;
+  using namespace Dune::Indices;
   const auto& fluxLocalFiniteElement     = localView.tree().child(_0).finiteElement();
   const auto& pressureLocalFiniteElement = localView.tree().child(_1).finiteElement();
 
@@ -159,7 +160,7 @@ void getVolumeTerm( const LocalView& localView,
   localRhs = 0;
 
   // Get set of shape functions for this element
-  using namespace Dune::TypeTree::Indices;
+  using namespace Dune::Indices;
   const auto& fluxLocalFiniteElement     = localView.tree().child(_0).finiteElement();
   const auto& pressureLocalFiniteElement = localView.tree().child(_1).finiteElement();
 
@@ -203,25 +204,23 @@ void getOccupationPattern(const Basis& basis,
 
   // A view on the FE basis on a single element
   auto localView = basis.localView();
-  auto localIndexSet = basis.localIndexSet();
 
   // Loop over all leaf elements
   for(const auto& element : elements(basis.gridView()))
   {
     // Bind the local FE basis view to the current element
     localView.bind(element);
-    localIndexSet.bind(localView);
 
     // Add element stiffness matrix onto global stiffness matrix
-    for (size_t i=0; i<localIndexSet.size(); i++)
+    for (size_t i=0; i<localView.size(); i++)
     {
       // The global index of the i-th local degree of freedom of the element 'e'
-      auto row = localIndexSet.index(i);
+      auto row = localView.index(i);
 
-      for (size_t j=0; j<localIndexSet.size(); ++j)
+      for (size_t j=0; j<localView.size(); ++j)
       {
         // The global index set of the j-th local degree of freedom of the element 'e'
-        auto col = localIndexSet.index(j);
+        auto col = localView.index(j);
 
         nb[row[0]][col[0]].add(row[1], col[1]);
       }
@@ -253,14 +252,12 @@ void assembleMixedPoissonMatrix(const Basis& basis,
 
   // A view on the FE basis on a single element
   auto localView = basis.localView();
-  auto localIndexSet = basis.localIndexSet();
 
   // A loop over all elements of the grid
   for(const auto& element : elements(gridView))
   {
     // Bind the local FE basis view to the current element
     localView.bind(element);
-    localIndexSet.bind(localView);
 
     // Now let's get the element stiffness matrix
     // A dense matrix is used for the element stiffness matrix
@@ -271,12 +268,12 @@ void assembleMixedPoissonMatrix(const Basis& basis,
     for (size_t i=0; i<elementMatrix.N(); i++)
     {
       // The global index of the i-th local degree of freedom of the element 'e'
-      auto row = localIndexSet.index(i);
+      auto row = localView.index(i);
 
       for (size_t j=0; j<elementMatrix.M(); j++ )
       {
         // The global index of the j-th local degree of freedom of the element 'e'
-        auto col = localIndexSet.index(j);
+        auto col = localView.index(j);
         matrix[row[0]][col[0]][row[1]][col[1]] += elementMatrix[i][j];
       }
     }
@@ -296,22 +293,20 @@ void assembleMixedPoissonRhs(const Basis& basis,
   auto localVolumeTerm = localFunction(Functions::makeGridViewFunction(volumeTerm, gridView));
 
   // set rhs to correct length -- the total number of basis vectors in the basis
-  typedef Dune::Functions::HierarchicVectorWrapper<VectorType, double> HierarchicVectorView;
-  HierarchicVectorView(rhs).resize(basis);
+  using Functions::istlVectorBackend;
+  istlVectorBackend(rhs).resize(basis);
 
   // Set all entries to zero
   rhs = 0;
 
   // A view on the FE basis on a single element
   auto localView = basis.localView();
-  auto localIndexSet = basis.localIndexSet();
 
   // A loop over all elements of the grid
   for(const auto& element : elements(gridView))
   {
     // Bind the local FE basis view to the current element
     localView.bind(element);
-    localIndexSet.bind(localView);
 
     // Now get the local contribution to the right-hand side vector
     BlockVector<FieldVector<double,1> > localRhs;
@@ -321,7 +316,7 @@ void assembleMixedPoissonRhs(const Basis& basis,
     for (size_t i=0; i<localRhs.size(); i++)
     {
       // The global index of the i-th vertex of the element 'e'
-      auto row = localIndexSet.index(i);
+      auto row = localView.index(i);
       rhs[row[0]][row[1]] += localRhs[i];
     }
   }
@@ -338,10 +333,10 @@ int main (int argc, char *argv[])
 
 #ifdef DIM2
   const int dim = 2;
-  std::array<int,dim> elements = {50, 50};
+  std::array<int,dim> elements = {{50, 50}};
 #else
   const int dim = 3;
-  std::array<int,dim> elements = {10, 10, 10};
+  std::array<int,dim> elements = {{10, 10, 10}};
 #endif
   typedef YaspGrid<dim> GridType;
   FieldVector<double,dim> l(1);
@@ -354,17 +349,21 @@ int main (int argc, char *argv[])
   //   Choose a finite element space
   /////////////////////////////////////////////////////////
 
-  using namespace Functions::BasisBuilder;
+  using namespace Functions::BasisFactory;
 
   const int k = 0;
 
   auto basis = makeBasis(
     gridView,
     composite(
-      rt<k, GeometryType::BasicType::cube>(),
-      pq<k>()
+      raviartThomas<k>(),
+      lagrange<k>()
     ));
 
+  using namespace Indices;
+  auto fluxBasis = Functions::subspaceBasis(basis, _0);
+  auto pressureBasis = Functions::subspaceBasis(basis, _1);
+
 
   /////////////////////////////////////////////////////////
   //   Stiffness matrix and right hand side vector
@@ -372,7 +371,8 @@ int main (int argc, char *argv[])
 
   typedef Matrix<BCRSMatrix<FieldMatrix<double,1,1> > > MatrixType;
   typedef BlockVector<BlockVector<FieldVector<double,1> > > VectorType;
-  typedef Dune::Functions::HierarchicVectorWrapper<VectorType, double> HierarchicVectorView;
+
+  using Functions::istlVectorBackend;
 
   MatrixType stiffnessMatrix;
   VectorType rhs;
@@ -394,7 +394,6 @@ int main (int argc, char *argv[])
 
   using namespace TypeTree::Indices;
   using BitVectorType = BlockVector<BlockVector<FieldVector<char,1> > >;
-  using HierarchicBitVectorView = Functions::HierarchicVectorWrapper<BitVectorType, char>;
 
   BitVectorType isTopBoundary;
   BitVectorType isLowerBoundary;
@@ -415,7 +414,7 @@ int main (int argc, char *argv[])
   };
 
   // Mark lower boundary.
-  auto lowerBoundaryIndicator = [&l] (Domain x)
+  auto lowerBoundaryIndicator = [] (Domain x)
   {
 #ifdef DIM2
     double isBoundary = x[dim-1] < 1e-8 ? x[0]: 0.0;
@@ -428,10 +427,10 @@ int main (int argc, char *argv[])
   VectorType isTopBoundaryTmp, isLowerBoundaryTmp;
 
   // Use double-valued interpolation and transfer to char-valued vectors.
-  interpolate(basis, Dune::TypeTree::hybridTreePath(_0), HierarchicVectorView(isTopBoundaryTmp),   topBoundaryIndicator);
-  interpolate(basis, Dune::TypeTree::hybridTreePath(_0), HierarchicVectorView(isLowerBoundaryTmp), lowerBoundaryIndicator);
-  HierarchicBitVectorView(isTopBoundary).resize(basis);
-  HierarchicBitVectorView(isLowerBoundary).resize(basis);
+  interpolate(fluxBasis, istlVectorBackend(isTopBoundaryTmp),   topBoundaryIndicator);
+  interpolate(fluxBasis, istlVectorBackend(isLowerBoundaryTmp), lowerBoundaryIndicator);
+  istlVectorBackend(isTopBoundary).resize(basis);
+  istlVectorBackend(isLowerBoundary).resize(basis);
   isTopBoundary = 0;
   isLowerBoundary = 0;
   for (size_t i=0; i<isTopBoundaryTmp[0].size(); i++)
@@ -440,8 +439,9 @@ int main (int argc, char *argv[])
     isLowerBoundary[0][i] = isLowerBoundaryTmp[0][i]!=0 ? 1: 0;
   }
 
-  interpolate(basis, Dune::TypeTree::hybridTreePath(_0), HierarchicVectorView(rhs), topFluxBC,   HierarchicBitVectorView(isTopBoundary));
-  interpolate(basis, Dune::TypeTree::hybridTreePath(_0), HierarchicVectorView(rhs), lowerFluxBC, HierarchicBitVectorView(isLowerBoundary));
+
+  interpolate(fluxBasis, istlVectorBackend(rhs), topFluxBC,   istlVectorBackend(isTopBoundary));
+  interpolate(fluxBasis,istlVectorBackend(rhs), lowerFluxBC, istlVectorBackend(isLowerBoundary));
 
   ////////////////////////////////////////////
   //   Modify Dirichlet rows
@@ -496,13 +496,13 @@ int main (int argc, char *argv[])
   using FluxRange = FieldVector<double,dim>;
   using PressureRange = double;
 
-  auto fluxFunction = Dune::Functions::makeDiscreteGlobalBasisFunction<FluxRange>(basis, TypeTree::hybridTreePath(_0), HierarchicVectorView(x));
-  auto pressureFunction = Dune::Functions::makeDiscreteGlobalBasisFunction<PressureRange>(basis, TypeTree::hybridTreePath(_1), HierarchicVectorView(x));
+  auto fluxFunction = Functions::makeDiscreteGlobalBasisFunction<FluxRange>(fluxBasis, istlVectorBackend(x));
+  auto pressureFunction = Functions::makeDiscreteGlobalBasisFunction<PressureRange>(pressureBasis, istlVectorBackend(x));
 
   //////////////////////////////////////////////////////////////////////////////////////////////
   //  Write result to VTK file
   //////////////////////////////////////////////////////////////////////////////////////////////
-  SubsamplingVTKWriter<GridView> vtkWriter(gridView,0);
+  SubsamplingVTKWriter<GridView> vtkWriter(gridView, Dune::refinementLevels(0));
   vtkWriter.addVertexData(fluxFunction, VTK::FieldInfo("flux", VTK::FieldInfo::Type::vector, dim));
   if (k==0)
     vtkWriter.addCellData(pressureFunction, VTK::FieldInfo("pressure", VTK::FieldInfo::Type::scalar, 1));
